include <p12F675.inc>
__CONFIG 0x3F84

CBLOCK 0x20
	W_TEMP : 1
	STATUS_TEMP : 1
	myFlags : 1
ENDC

swapwf MACRO REG
	xorwf REG,F
	xorwf REG,W
	xorwf REG,F
ENDM

#DEFINE deltaT myFlags,0

; Vetor de Inicio de programa.
org 0
goto _configuration

; Interrupo
org 4
	movwf   W_TEMP            ; save off current W register contents
	movf	STATUS,w          ; move status register into W register
	movwf	STATUS_TEMP       ; save off contents of STATUS register
	; -----------------------------------
	; ROTINA DE TRATAMENTO DA INTERRUPO
	; -----------------------------------

	btfsc deltaT
	goto reiniciaCont
	
	bsf deltaT	; Seta a Flag para dizer que est lendo!
	clrf TMR1L
	clrf TMR1H
	bsf T1CON, TMR1ON	; Liga o Timer 1
	goto retorna_ISR

reiniciaCont:
	bcf deltaT
	bcf T1CON, TMR1ON	; Para o Timer 1
	swapwf TMR1H
	sublw 0x1C		; 8 us - 1 pulso Timer  | W - L
	btfss STATUS, C	; Houve estouro?
	goto ehMaior
	goto ehMenor
	
ehMenor:
	bcf GPIO, GPIO1
	goto retorna_ISR

ehMaior
	bsf GPIO, GPIO1
;	btfsc GPIO, GPIO0
;	goto apaga
;
;acende:
;	bsf GPIO, GPIO0
;	bsf GPIO, GPIO1
;	bsf GPIO, GPIO4
;	bsf GPIO, GPIO5
;	nop
;	nop
;	goto retorna_ISR
;
;apaga:
;	bcf GPIO, GPIO0
;	bcf GPIO, GPIO1
;	bcf GPIO, GPIO4
;	bcf GPIO, GPIO5
;	nop
;	nop
;
retorna_ISR:
	call _changeEdge		  ; muda a Borda da Interrupo.
	BANKSEL GPIO
	bcf INTCON, INTF		  ; Limpando a Flag da Mudana da Porta.
	movf    STATUS_TEMP,w     ; retrieve copy of STATUS register
	movwf	STATUS            ; restore pre-isr STATUS register contents
	swapf   W_TEMP,f
	swapf   W_TEMP,w          ; restore pre-isr W register contents
	retfie                    ; return from interrupt

_configuration:
	clrw
	clrf myFlags
	clrf TMR1L
	clrf TMR1H
	BANKSEL OSCCAL
	call 0x3FF		; Calibrando o Clock para 4Mhz
	movwf OSCCAL	

	clrf PIE1		; Nenhuma Interrupo de Perifricos habilitadas.	

	movlw b'00000111'	; 1 - Input e 0 - Output
	movwf TRISIO
	
	movlw b'01010011'	; 1 - Analgico e 0 - Digital I/O
	movwf ANSEL

	movlw b'01001000'			; PreScaler pro WatchDog, se quiser TIMER0<3> zerar BIT
	movwf OPTION_REG
	
	BANKSEL CMCON
	movlw 0x07		; Desligando os Comparadores, para menor consume de energia;
	movwf CMCON

	movlw b'01000001' ; Vref < AN1 >, CHANEL 0,
	movwf ADCON0

	movlw b'00110000'	; PreScale para 1:8 ; Mandando Ver!
	movwf T1CON
	
	;Verificar a necessidade de interrupo por tempo!
	movlw b'10010000' ; GIE + GP2/INT
;	movlw b'00010000' ; GIE + GP2/INT DESABILITEI!
	movwf INTCON


loop:
;	nop
;	goto loop
	nop
;	call CAD
;	swapwf ADRESH
	sublw .512
	btfss STATUS, C	; Houve estouro?
	goto semPerigo
	goto comPerigo

semPerigo:
	bsf GPIO, GPIO3
	goto loop

comPerigo:
	bsf GPIO, GPIO3
	goto loop
	


; Mudar a Borda, s Funciona para GPIO2
_changeEdge:
	BANKSEL OPTION_REG
	btfss OPTION_REG, INTEDG
	goto swaptoRise
	bcf OPTION_REG, INTEDG
	return
swaptoRise:
	bsf OPTION_REG, INTEDG
	return
	
;CAD:
;	bsf ADCON0, GO ; Inicia a converso A/D
;	btfsc ADCON0, GO
;	goto $-1
;	return
	
	

org 0x3FF
dt 0xFF		; Data table que retorna o valor do Clock em W, tipo "retlw"

end