#include "P16F876A.INC"

__CONFIG 0x3F33

CBLOCK 0x20
    W_TEMP : 1
    STATUS_TEMP : 1
    myFlags : 1
    direcao : 1
    cont1    : 1
    cont2     : 1
    contNop    : 1
ENDC

#define Bateria myFlags, 0
#define Sensores myFlags, 1
#define qtde_NOPS d'12'
#define FE direcao,0
#define FD direcao,1
#define TE direcao,2
#define TD direcao,3

org 0
goto setup

org 4
    ; Saving the context
    movwf W_TEMP ; Copy W to TEMP register
    swapf STATUS,W ; Swap status to be saved into W
    clrf STATUS ; cbank 0, regardless of current bank, Clears IRP,RP1,RP0
    movwf STATUS_TEMP ; Save status to bank zero STATUS_TEMP register

    ;    -----------------------------------------------
    ;    ISR - INTERRUPT SERVICE ROUTINE BEGINS HERE!!!
    ;    -----------------------------------------------

    btfsc INTCON, RBIF    ;    SENSORES DIANTEIROS E TRASEIROS
    goto ISR_Sensores
ponto1:

    btfsc INTCON, INTF    ;    SENSOR DA BATERIA
    goto ISR_Bateria
ponto2:

    btfsc Sensores
    goto funcao_Sensores
ponto3:

    ;    -----------------------------------------------
    ;    ISR - INTERRUPT SERVICE ROUTINE ENDS HERE!!!
    ;    -----------------------------------------------

    ; Recovering the context
    swapf STATUS_TEMP,W ; Swap STATUS_TEMP register into W
                        ;(sets bank to original state)
    movwf STATUS ; Move W into STATUS register
    swapf W_TEMP,F ; Swap W_TEMP
    swapf W_TEMP,W ; Swap W_TEMP into W
    retfie


setup:
    BANKSEL TRISC
    movlw 0xFF
    movwf TRISB        ; PORTAS B S entrada
    movlw 0x0F        ; <3..0> Entrada | <4..7> Sada
    movwf TRISC        ; PORTAS C S sada Todas
    BANKSEL PORTC
    clrf PORTC        ; aqui precisa 0.
    clrf direcao    ; limpando a direo
    clrf myFlags    ; limpa as flags
    movlw B'10011000'    ; Habilita interrupes, RBCHANGE e INT/RB0
    movwf INTCON

start:
    btfsc Bateria
    goto funcao_Bateria
ponto4:

    ; INTELIGENCIA ARTIFICIAL CARRINHO RODANDO.
    movlw b'00000000'        ; 00 00 xxxx
	movwf PORTC
	call wait_Time

    movlw b'00010000'        ; 00 01 xxxx
	movwf PORTC
	call wait_Time
 
    goto start



    ;    -----------------------------------------------
    ;    ISR - INTERRUPT SERVICE ROUTINE BEGIN HERE!!!
    ;    -----------------------------------------------
ISR_Sensores:
    movfw PORTB            ; Tem que ler a PORTB para que possa ser zera a flag!
    bcf INTCON, RBIF    ; Limpando a flag antes de mais nada.

    btfss PORTB, 4        ; 
    bsf direcao, 0        ; Foi o sensor 1
    btfss PORTB, 5        ; 
    bsf direcao, 1        ; Foi o sensor 2
    btfss PORTB, 6        ; 
    bsf direcao, 2        ; Foi o sensor 3
    btfss PORTB, 7        ; 
    bsf direcao, 3        ; Foi o sensor 4

    btfss PORTB, 4
    bsf Sensores
    btfss PORTB, 5
    bsf Sensores
    btfss PORTB, 6
    bsf Sensores
    btfss PORTB, 7
    bsf Sensores
        
    goto ponto1

ISR_Bateria:
    bcf INTCON, INTF    ; Limpando a flag antes de mais nada.
    bsf Bateria    
    goto ponto2

    ;    -----------------------------------------------
    ;    ISR - INTERRUPT SERVICE ROUTINE ENDS HERE!!!
    ;    -----------------------------------------------


    ;    -----------------------------------------------
    ;        FUNES AUXILIARES COMEAM AQUI
    ;    -----------------------------------------------

funcao_Bateria:
    ;
    ; Aqui ele s pega do PIC pequeno e vai mandando para porta C, controle do CARRINHO
    bcf Bateria
    goto ponto4

funcao_Sensores:
	clrw
	movwf PORTC
    bcf Sensores
	btfsc FE
	movlw b'10010000'
	btfsc FD
	iorlw b'10010000'
	btfsc TE
	iorlw b'00110000'
	btfsc TD
	iorlw b'00110000'
	movwf PORTC
    call wait_Time
	clrf PORTC
	clrf direcao
    goto ponto3


wait_Time:
    movlw 0x80
    movwf cont2
backT2:
    movlw 0xFF
    movwf cont1
backT1:
	bsf PORTC, 6
	bcf PORTC, 6
	movlw qtde_NOPS
	movwf contNop
    decfsz contNop, F
	goto $-1
    decfsz cont1, F
    goto backT1
    decfsz cont2, F
    goto backT2
	return
    
    ;    -----------------------------------------------
    ;        FUNES AUXILIARES TERMINAM AQUI AQUI
    ;    -----------------------------------------------

END